#ifndef DGX_ASSET_H
#define DGX_ASSET_H

#define ASSET_TYPE_MOL		0
#define ASSET_TYPE_MTD		1
#define ASSET_TYPE_ATD		2
#define ASSET_TYPE_CXC		3

#define MOL_COLMAP_ELEM_SIZE	20
#define MOL_AREATRIG_ELEM_SIZE	16
#define MOL_LAYERS_MAX		256
#define MOL_LAYER_METADATA_SIZE	8

#define MOL_LATEST_VERSION	1
#define MTD_LATEST_VERSION	5
#define ATD_LATEST_VERSION	1
#define CXC_LATEST_VERSION	0

#define CXC_PARENT_NONE		0
#define CXC_PARENT_MTD		1
#define CXC_PARENT_ATD		2

#define XASS_FLAG_STP		(1<<0)
#define XASS_FLAG_WIREFRAME	(1<<1)
#define XASS_FLAG_TEXTURED	(1<<2)
#define XASS_FLAG_LIGHTING	(1<<3)
#define XASS_FLAG_GOURAUD	(1<<4)
#define XASS_FLAG_PNTLITE	(1<<5)
#define XASS_FLAG_CULLING	(1<<6)
#define XASS_FLAG_BACKFACE_CULL	(1<<7)

#include <engine/psx.h>
#include <engine/3d/shapes.h>
#include <engine/3d/camera.h>
#include <engine/3d/entity.h>

/*
 * APBuf is the POLYGON section. These
 * are buffers that store the actual polygons
 * stored in memory, their values are given by
 * what is stored in the asset file.
 */
struct APBuf /* Asset Polygon Buffer */
{
	POLY_F3		*p_polyf3;
	POLY_F4		*p_polyf4;
	POLY_FT3	*p_polyft3;
	POLY_FT4	*p_polyft4;
	POLY_G3		*p_polyg3;
	POLY_G4		*p_polyg4;
	POLY_GT3	*p_polygt3;
	POLY_GT4	*p_polygt4;
	LINE_F2		*p_linef2;
};

struct AssetMOL
{
	/* HEADER section */
	struct
	{
		unsigned char	id;
		unsigned char	version;
		unsigned char	tile_w;
		unsigned char	tile_h;
	}HEADER;

	/* COLMAP section */
	struct
	{
		unsigned int	bnum;

		struct COLMAP_Collider
		{
			unsigned int	shape;
			unsigned int	x;
			unsigned int	y;
			unsigned int	w;
			unsigned int	h;
		}*p_elem;
	}COLMAP;

	/* AREATRIG section */
	struct
	{
		unsigned int	bnum;

		struct AREATRIG_Trigger
		{
			unsigned int	x;
			unsigned int	y;
			unsigned int	w;
			unsigned int	h;
		}*p_elem;
	}AREATRIG;

	/* LYRHEAD section */
	struct
	{
		unsigned char	numlayers;
		unsigned char	reserved[3];

		struct LYRHEAD_Layer
		{
			/* metadata */
			unsigned int	bnum;
			unsigned char	width;
			unsigned char	height;
			unsigned char	is_sparse;
			unsigned char	reserved;

			/* data */
			struct LayerCoordinatedList
			{
				unsigned char	val;
				unsigned char	row;
				unsigned char	col;
			}*p_coo;

			struct LayerMatrix
			{
				unsigned char	val;
			}*p_matrix;
		}layer[256];
	}LYRHEAD;
};

/*
 * the AVInd struct corresponds to
 * the "type" and "index" of each POLYGON
 * in an MTD/ATD file. RGB and UV data is
 * written directly to the appropriate p_poly*
 * buffer for each polygon.
 */
struct AVInd /* Asset Vertex Index */
{
	unsigned int	type; /* important for matching it with the POLYGON section */
	/* "index 0" to "index 3" -- indices to p_vertex array */
	unsigned short	v[4];
	unsigned short	n[4];
};

struct AssetMTD
{
	unsigned int		id;
	unsigned short		version, flag;
	unsigned short		nvert, npoly;

	VECTOR			*p_vertex;	/* raw vertex data */
	VECTOR			*p_normal;	/* raw normal data */

	struct APBuf		apb;

	struct AVInd		*p_indices;

	/* other */
	unsigned short		tpage, clut;			/* only used if the model is textured */
	unsigned short		linef2_cnt;			/* number of LINE_F2s */
	unsigned short		polyf3_cnt, polyf4_cnt;	
	unsigned short		polyft3_cnt, polyft4_cnt;
	unsigned short		polyg3_cnt, polyg4_cnt;
	unsigned short		polygt3_cnt, polygt4_cnt;

	struct Entity3D		ent;
	SVECTOR			rotmod2;
	int			rotm2;
};

struct AssetATD
{
	/* HEADER section */
	unsigned int	id;
	unsigned short	version, flag;
	unsigned short	nvert, npoly;
	unsigned char	nkey, fps;

	enum INTR_Types
	{
		INSTANT		= 0x00,
		LINEAR		= 0x01
	}intr;

	unsigned char	unused;

	/*
	 * VERTEX section. These are the
	 * vertices stored in memory, their
	 * values given by what is stored in
	 * the ATD file.
	 */
	int		vbnum;
	struct ATD_VERTEX
	{
		int		*p_keyframe_framenum;
		VECTOR		*p_keyframe_vector;
	}*p_vertex;

	VECTOR			*p_normal;

	struct APBuf		apb;

	struct AVInd		*p_indices;

	/* other */
	unsigned short		tpage, clut;			/* only used if the model is textured */
	unsigned short		linef2_cnt;			/* number of LINE_F2s */
	unsigned short		polyf3_cnt, polyf4_cnt;		/* number of POLYF3s and POLYF4s */
	unsigned short		polyft3_cnt, polyft4_cnt;	/* number of POLYFT3s and POLYFT4s */
	unsigned short		polyg3_cnt, polyg4_cnt;
	unsigned short		polygt3_cnt, polygt4_cnt;

	struct Entity3D		ent;
	SVECTOR			rotmod2;
	int			rotm2;

	/* ATD-specific */
	int			timeline_cursor;
	int			displayed_keyframe;
};

struct AssetCXC
{
	unsigned int		id;
	unsigned short		reserved, version;

	struct
	{
		unsigned int		bnum;
		unsigned short		nquads, ncuboids;

		/* primitive buffers */
		struct AssQuad		*p_quad;
		struct AssCuboid	*p_cuboid;
	}COLMAP;

	struct
	{
		unsigned int		bnum;
		unsigned short		reserved, ncuboids;

		/* primitive buffers */
		struct AssCuboid	*p_cuboid;
	}AREATRIG;

	struct Entity3D		ent;

	/* parent setting & data must be set manually */
	int			parent;
	void			*p_parent;			/* pointer to MTD/ATD/etc.. */
};

#endif

